#ifndef SONG_COLLECTION
#define SONG_COLLECTION




#include <vector>
#include <qobject.h>
#include "song.h"







class tProgress;




class tSongCollection : public QObject
{
    Q_OBJECT

    typedef hash_map<tUniqueId, tSong *> tUniqueIdMap;
    typedef hash_map<tFilename, tSong *, hash_QString> tFilenameMap;

    tSongList SongList;
    tSongList Orphans;
    tSongList New;

    tUniqueIdMap UniqueIdMap;
    tFilenameMap FilenameMap;

    tUniqueId NextUniqueId;

  public:

    typedef tSongList::const_iterator const_iterator;

    tSongCollection();

    const_iterator begin() const
    {
      return SongList.begin();
    }
    const_iterator end() const
    {
      return SongList.end();
    }

  public slots:
    void readTags( tProgress *progress,
	const QString &progresstext = "Reading tags...", 
	tSongList *list = NULL );
    void rereadTags( tProgress *progress );
    void scan( tDirectoryList const &dl, tProgress *progress );
    void insert( tSong *song, bool loaded );
    void remove( tSong *song );
    void clear();

    void noticeSongModified( tSong *song );

    void beginBulkChange();
    void endBulkChange();

  public:
    QDomNode serialize( QDomDocument &doc );

    tSong *getByFilename( tFilename const &filename );
    tSong *getByUniqueId( tUniqueId const &uid );
    bool isValid( tSong *song );

  signals:
    void notifyBeginBulkChange();
    void notifyEndBulkChange();
    void notifyAddition( tSong *song );
    void notifyDeletion( tSong *song );
    void notifyLoad( tSong *song );
    void notifySongModified( tSong *song );

  private:
    tUniqueId nextUniqueId();
    void scanSingleDirectory( tFilename const &dir, tProgress *progress );

    // make this type noncopyable
    tSongCollection( const tSongCollection &src );
    tSongCollection &operator=( const tSongCollection &src );
};




class tSongCollectionScopedBulkChange
{
  tSongCollection &Collection;
  public:
    tSongCollectionScopedBulkChange( tSongCollection &collection )
      : Collection( collection )
    {
      Collection.beginBulkChange();
    }
    ~tSongCollectionScopedBulkChange()
    {
      Collection.endBulkChange();
    }

};




#endif
