#include <stdexcept>
#include <qapplication.h>

#include "song_set_tree.h"




tSongSetNode::tSongSetNode( tSongSet *data )
    : Parent( NULL ), Data( data )
{}




tSongSetNode::~tSongSetNode()
{
  clear();
  delete Data;
}




tSongSetNode *tSongSetNode::parent()
{
  return Parent;
}




void tSongSetNode::addChild( tSongSetNode *node )
{
  node->Parent = this;
  ChildrenList.push_back( node );
}




void tSongSetNode::insertChild( tSongSetNode *node, iterator it )
{
  node->Parent = this;
  ChildrenList.insert( it, node );
}




void tSongSetNode::removeChild( tSongSetNode *node )
{
  node->Parent = NULL;
  FOREACH( first, ChildrenList, tChildrenList )
  if ( node == *first )
    ChildrenList.erase( first );
}




void tSongSetNode::clear()
{
  FOREACH( first, ChildrenList, tChildrenList )
  delete * first;
  ChildrenList.clear();
}




QString tSongSetNode::qualifiedName()
{
  if ( Parent && Parent->Parent )
    return Parent->qualifiedName() + "/" + name();
  else
    return name();
}




tSongSetNode *tSongSetNode::resolve( const QString &path )
{
  int slash_index = path.find( '/' );

  QString local_component,remainder;
  if ( slash_index == -1 )
    local_component = path;
  else
  {
    local_component = path.left( slash_index );
    remainder = path.mid( slash_index + 1 );
  }

  FOREACH_CONST( first, ChildrenList, tChildrenList )
  {
    if ( local_component == (*first)->name() )
    {
      if ( slash_index == -1 )
	return *first;
      else
	return (*first)->resolve( remainder );
    }
  }
  return NULL;
}




bool tSongSetNode::hasParent( tSongSetNode *node )
{
  if ( Parent == node )
    return true;

  if ( Parent )
    return Parent->hasParent( node );
  else
    return false;
}




QString const &tSongSetNode::name()
{
  return Name;
}




void tSongSetNode::setName( QString const &name )
{
  if ( name.find( '/' ) == -1 )
    Name = name;
  else
    runtimeError( qApp->translate( "ErrorMessages", "Slashes may not occur in playlist names." ) );
}




tSongSet *tSongSetNode::data()
{
  return Data;
}




void tSongSetNode::setData( tSongSet *data )
{
  if ( data )
    delete Data;
  Data = data;
}




QDomNode tSongSetNode::serialize( QDomDocument &doc )
{
  QDomElement result = doc.createElement( "song_set_node" );
  result.setAttribute( "name", Name );

  if ( Data )
    result.appendChild( Data->serialize( doc ) );

  FOREACH( first, ChildrenList, tChildrenList )
    result.appendChild( ( *first ) ->serialize( doc ) );
  return result;
}

