#ifndef MAINWIN
#define MAINWIN




#include <memory>
#include <qtimer.h>
#include <qlistview.h>

#include "designer/mainwin_ui.h"
#include "database/database.h"
#include "ui/prefs.h"
#include "ui/song_set_view.h"




class tSongSetNodeListViewItem;
class tProgress;




class tMainWindow : public tMainWindowBase
{
    Q_OBJECT;

    auto_ptr<tDatabase> Database;

    tSongSet AllSongsSet;
    tAllSongsViewManager AllSongsView;
    tEditableSongSetViewManager SongSetEditor;

    auto_ptr<tCriterion> SearchCriterion;
    QSettings		Settings;
    tPreferences	Preferences;

    tFilename CurrentFilename;
    bool FilenameValid;

    tFilename CurrentSongFilename;
    double CurrentSongPlayingSeconds;
    QTimer CurrentSongCheckerTimer;

    tProgress *CurrentSearchProgress;

  public:
    tMainWindow();
    virtual ~tMainWindow();

    void setDatabase( tDatabase *db );

    void closeEvent( QCloseEvent *event );

    void setupListView( tSongSetViewManager &mgr, bool save_before );
    void updateWindowCaption();
    void updateAll();

    tDatabase &database()
    {
      return *Database;
    }
    tPreferences &preferences()
    {
      return Preferences;
    }
    QSettings &settings()
    {
      return Settings;
    }

  public slots:
    void rescan();
    void rescan( tDatabase *db );
    void rereadTags();
  
    void fileNew();
    void fileOpen();
    void saveDatabase( const QString &name );
    void fileSave();
    bool fileSaveWithResult();
    void fileSaveAs();
    bool fileSaveAsWithResult();
    void filePreferences();
    
    void helpAbout();
    void help();

    void addSongSet( tSongSetNode *node );
    void addSongSet();
    void duplicateSongSet();
    void removeSongSet();
    void editMultilineSongSetCriterion();
    void slotDropSongSetNode( const QString &node, tSongSetNode *onto );
    void renameSongSet( QListViewItem *item, int col, const QString &text );
    void songSetSelectionChanged();
    void songSetCriterionChanged();

    void add( tSongList const &songlist );

    void searchChanged();
    void searchChanged( const QString &text )
    {
      searchChanged();
    }

    void buildOverviewTree();
    void overviewSelectionChanged( QListViewItem *item );

    void checkCurrentlyPlayingSong();

  private:
    void updateSongSetTree( bool keep_selection, bool keep_scroll );
    tSongSetNode *nodeFromItem( QListViewItem *item );
    tSongSetNode *currentNode();
    tSongSet *currentSongSet();
    void insertAtPosition( QListViewItem *parent, QListViewItem *item, tIndex pos );

    QListViewItem *createSongSetNodeItem( tSongSetNode *node, QListViewItem *parent,
	tSongSetNode *selected_node, tSongSetNodeListViewItem *&selected_item );

    void loadSplitterAppearance( QSplitter *splitter, QSettings &settings, QString const &where );
    void saveSplitterAppearance( QSplitter *splitter, QSettings &settings, QString const &where );
};




#endif
