#include "ui/overview.h"
#include "database/database.h"
#include <algorithm>




// tOverviewItem --------------------------------------------------------------
tOverviewItem::tOverviewItem( QListView *view,  QListViewItem *after, const QString &label, const QString &criterion )
: QListViewItem( view, after, label ), Criterion( criterion )
{ 
}




tOverviewItem::tOverviewItem( QListViewItem *parent, QListViewItem *after, const QString &label, const QString &criterion )
: QListViewItem( parent, after, label ), Criterion( criterion )
{
}





// tExpandableOverviewItem --------------------------------------------------------------
tExpandableOverviewItem::tExpandableOverviewItem( QListView *view,  QListViewItem *after, const QString &label, const QString &criterion )
: tOverviewItem( view, after, label, criterion )
{ 
  setExpandable( true );
  HasExpanded = false;
}




tExpandableOverviewItem::tExpandableOverviewItem( QListViewItem *parent, QListViewItem *after, const QString &label, const QString &criterion )
: tOverviewItem( parent, after, label, criterion )
{
  setExpandable( true );
  HasExpanded = false;
}




void tExpandableOverviewItem::setOpen( bool o )
{
  if ( !HasExpanded && o )
  {
    listView()->setUpdatesEnabled( false );
    addExpansion();
    listView()->setUpdatesEnabled( true );
    HasExpanded = true;
  }
  tOverviewItem::setOpen( o );
}




void tExpandableOverviewItem::addExpansion()
{
}




// tArtistsOverviewItem -------------------------------------------------------
tArtistsOverviewItem::tArtistsOverviewItem( tDatabase &database, QListView *view,  QListViewItem *after, const QString &label, const QString &criterion )
: tExpandableOverviewItem( view, after, label, criterion ), 
  Database( database )
{ 
}




tArtistsOverviewItem::tArtistsOverviewItem( tDatabase &database, QListViewItem *parent, QListViewItem *after, const QString &label, const QString &criterion )
: tExpandableOverviewItem( parent, after, label, criterion ),
  Database( database )
{ 
}




void tArtistsOverviewItem::addExpansion()
{
  tSongList songs( Database.SongCollection.begin(), Database.SongCollection.end() );
  ::sort( songs.begin(), songs.end(), less_song( FIELD_ARTIST ) );
  QListViewItem *last_item = NULL;
  QString last_entry = QString::null;

  int artist_count = 0;

  tSongList::iterator first = songs.begin(), last = songs.end();
  while ( first != last )
  {
    QString this_nonnormalized_entry = (*first)->artist();
    QString this_entry = this_nonnormalized_entry.stripWhiteSpace().lower();
    if ( this_nonnormalized_entry == "" )
      this_nonnormalized_entry = "<none>";

    tSongList this_songs;
    while ( first != last && this_entry == (*first)->artist().stripWhiteSpace().lower() )
      this_songs.push_back( *first++ );

    ::sort( this_songs.begin(), this_songs.end(), less_song( FIELD_ALBUM ) );
    QListViewItem *entry_item = new tOverviewItem( this, last_item, this_nonnormalized_entry, QString( "~artist(complete:%1)" ).arg( quoteString( this_entry ) ) );

    int album_count = 0;
    int song_count = 0;

    QString last_subentry = QString::null;
    QListViewItem *last_subitem = NULL;

    FOREACH( sub_first, this_songs, tSongList )
    {
      QString this_nonnormalized_subentry = (*sub_first)->album();
      QString this_subentry = this_nonnormalized_subentry.stripWhiteSpace().lower();
      if ( this_nonnormalized_subentry == "" )
	this_nonnormalized_subentry = "<none>";

      if ( this_subentry != last_subentry )
      {
	if ( last_subitem )
	{
	  last_subitem->setText( 0, last_subitem->text( 0 ) + QString(" (%1)").arg( song_count ) );
	  song_count = 0;
	}

	last_subitem = new tOverviewItem( entry_item, last_subitem, this_nonnormalized_subentry, 
	    QString( "~artist(complete:%1)&~album(complete:%2)" )
	    .arg( quoteString( this_entry ) )
	    .arg( quoteString( this_subentry ) ) );
	album_count++;
      }

      last_subentry = this_subentry;
      song_count++;
    }
    if ( last_subitem )
      last_subitem->setText( 0, last_subitem->text( 0 ) + QString(" (%1)").arg( song_count ) );

    entry_item->setText( 0, this_nonnormalized_entry + QString( " (%1)").arg( album_count ) );

    last_item = entry_item;
    artist_count++;
  }

  setText( 0, text( 0 )+QString( " (%1)").arg( artist_count ) );
}




// tAlbumOverviewItem -------------------------------------------------------
tAlbumOverviewItem::tAlbumOverviewItem( tDatabase &database, QListView *view,  QListViewItem *after, const QString &label, const QString &criterion )
: tExpandableOverviewItem( view, after, label, criterion ), 
  Database( database )
{ 
}




tAlbumOverviewItem::tAlbumOverviewItem( tDatabase &database, QListViewItem *parent, QListViewItem *after, const QString &label, const QString &criterion )
: tExpandableOverviewItem( parent, after, label, criterion ),
  Database( database )
{ 
}




void tAlbumOverviewItem::addExpansion()
{
  tSongList songs( Database.SongCollection.begin(), Database.SongCollection.end() );
  ::sort( songs.begin(), songs.end(), less_song( FIELD_ALBUM ) );

  int album_count = 0;
  int song_count = 0;

  QListViewItem *last_item = NULL;
  QString last_entry = QString::null;
  FOREACH( first, songs, tSongList )
  {
    QString this_nonnormalized_entry = (*first)->album();
    QString this_entry = this_nonnormalized_entry.stripWhiteSpace().lower();
    if ( this_nonnormalized_entry == "" )
      this_nonnormalized_entry = "<none>";

    if ( this_entry != last_entry )
    {
      if ( last_item )
      {
	last_item->setText( 0, last_item->text( 0 ) + QString(" (%1)").arg( song_count ) );
	song_count = 0;
      }

      last_item = new tOverviewItem( this, last_item, this_nonnormalized_entry, QString( "~album(complete:%1)" ).arg( quoteString( this_entry ) ) );
      album_count++;
    }
    last_entry = this_entry;
    song_count++;
  }

  if ( last_item )
    last_item->setText( 0, last_item->text( 0 ) + QString(" (%1)").arg( song_count ) );

  setText( 0, text( 0 )+QString( " (%1)").arg( album_count ) );
}





// tGenreOverviewItem -------------------------------------------------------
tGenreOverviewItem::tGenreOverviewItem( tDatabase &database, QListView *view,  QListViewItem *after, const QString &label, const QString &criterion )
: tExpandableOverviewItem( view, after, label, criterion ), 
  Database( database )
{ 
}




tGenreOverviewItem::tGenreOverviewItem( tDatabase &database, QListViewItem *parent, QListViewItem *after, const QString &label, const QString &criterion )
: tExpandableOverviewItem( parent, after, label, criterion ),
  Database( database )
{ 
}




void tGenreOverviewItem::addExpansion()
{
  tSongList songs( Database.SongCollection.begin(), Database.SongCollection.end() );

  ::sort( songs.begin(), songs.end(), less_song( FIELD_GENRE ) );
  QListViewItem *last_item = NULL;
  QString last_entry = QString::null;
  int song_count = 0;
  int genre_count = 0;

  FOREACH( first, songs, tSongList )
  {
    QString this_nonnormalized_entry = (*first)->genre();
    QString this_entry = this_nonnormalized_entry.stripWhiteSpace().lower();
    if ( this_nonnormalized_entry == "" )
      this_nonnormalized_entry = "<none>";
    if ( this_entry != last_entry )
    {
      if ( last_item )
      {
	last_item->setText( 0, last_item->text( 0 ) + QString(" (%1)").arg( song_count ) );
	song_count = 0;
      }

      last_item = new tOverviewItem( this, last_item, this_nonnormalized_entry, QString( "~genre(complete:%1)" ).arg( quoteString( this_entry ) ) );
      genre_count++;
    }
    last_entry = this_entry;
    song_count++;
  }

  if ( last_item )
    last_item->setText( 0, last_item->text( 0 ) + QString(" (%1)").arg( song_count ) );

  setText( 0, text( 0 )+QString( " (%1)").arg( genre_count ) );
}
