#include <qdragobject.h>
#include <qurl.h>

#include "database/song_set_tree.h"
#include "ui/set_list_view.h"
#include "ui/mainwin.h"

#include "set_list_view.moc"




// tSongSetNodeListViewItem ---------------------------------------------------
tSongSetNodeListViewItem::tSongSetNodeListViewItem( QListView *view, tSongSetNode *node )
: QListViewItem( view, node->name() ), Node( node )
{
  setRenameEnabled( 0, true );
  setDropEnabled( true );
  setDragEnabled( true );
}




tSongSetNodeListViewItem::tSongSetNodeListViewItem( QListViewItem *item, tSongSetNode *node )
    : QListViewItem( item, node->name() ), Node( node )
{
  setRenameEnabled( 0, true );
  setDropEnabled( true );
  setDragEnabled( true );
}




tSongSetNode *tSongSetNodeListViewItem::node()
{
  return Node;
}




bool tSongSetNodeListViewItem::acceptDrop( const QMimeSource *mime ) const
{
  return QUriDrag::canDecode( mime );
}




void tSongSetNodeListViewItem::dropped( QDropEvent *event )
{
  if ( QUriDrag::canDecode( event ) )
  {
    QStrList uri_list;
    QUriDrag::decode( event, uri_list );

    tSongSet *set = Node->data();

    vector<tFilename> filenames;
    char *uri = uri_list.first();
    while ( uri )
    {
      QString filename = QUriDrag::uriToLocalFile( uri );
      if ( !filename.isNull() )
      {
	try
	{
	  set->add( filename );
	}
	catch ( exception &ex )
	{
	  qWarning( "%s", ex.what() );
	}
      }
      else
      {
	QUrl url( uri );
	if ( url.protocol() == "madmanset" && event->source() == listView() )
	  emit dropNode( url.path().mid(1), Node );
      }
      uri = uri_list.next();
    }
  }
}




// tSetListView ---------------------------------------------------------------
tSetListView::tSetListView( QWidget *parent, const char *name, WFlags f )
: super( parent, name, f )
{
  addColumn( "Set" );
  setRootIsDecorated( true );

}




QDragObject *tSetListView::dragObject()
{
  tSongSetNodeListViewItem *item =
    (tSongSetNodeListViewItem *)( currentItem() );

  if ( item == NULL )
    return NULL;

  QUrl url;
  url.setProtocol( "madmanset" );
  url.setHost( "localhost" );
  url.setPath( item->node()->qualifiedName() );

  QStrList uri_list;
  uri_list.append( strdup( url.toString( true ).latin1() ) );
  return new QUriDrag( uri_list, this );
}
