#ifndef SONG_SET_VIEW
#define SONG_SET_VIEW




#include <qpopupmenu.h>
#include <qlistview.h>
#include <qsettings.h>
#include "database/song_set.h"
#include "ui/accel_list_view.h"




class tPreferences;




typedef vector<int> tIndexList;




class tSongSetViewManager;

class tSongListViewItem : public QObject, public QListViewItem
{
    Q_OBJECT;
    tSong *Song;
    tSongSetViewManager &Manager;
    float CriterionMatch;

  public:
    tSongListViewItem( tSongSetViewManager &mgr, tSong *song, float match, QListViewItem *after );
    void setRenameEnabledInternal( int field, bool value );

    tSong *song();
    QString text ( int column ) const;

  public slots:
    void update();
    void noticeSongModified( tSong *song );
};




class tSongListView : public tAcceleratorListView 
{
    Q_OBJECT;

  public:
    tSongListView( QWidget *parent = 0, const char *name = "", WFlags f = 0 );

    tSong *highlightedSong();
    void getSelection( tSongList &songlist );
    void getSelection( tIndexList &indexlist );

    void setSongDropEnable( bool enabled );

  protected:
    void contentsDragEnterEvent( QDragEnterEvent *event );
    void contentsDragMoveEvent( QDragMoveEvent *event );
    void contentsDropEvent( QDropEvent *event );

    QDragObject *dragObject();

  signals:
    void songsDropped( bool form_same_widget, tSong *onto, vector<tFilename> &songs );
};




class tSongRetag
{
  public:
    QString	NewText;
    tSongField	Column;

    void executeOn( tSong *song );
};




class tDeferredSongRetag : public QObject
{
    Q_OBJECT;

  public:
    tSong	*Song;
    tSongRetag	Retag;

  public slots:
    void execute();
};




class tMainWindow;

class tSongSetViewManager : public QObject
{
    Q_OBJECT;

  protected:
    tSongListView &ListView;
    tSongSet *SongSet;
    auto_ptr<QPopupMenu> Menu;

    tMainWindow &MainWindow;

    bool LastRetagValid;
    tSongRetag LastRetag;

    bool DoDisplaySort;
    tSongField SortField;
    bool Ascending;

    vector<int> FieldToColumn;
    vector<int> ColumnToField;

    QString PreferencesPrefix;

    hash_map<int,tFilename> MenuIdToPluginFilenameMap;

  public:
    tSongSetViewManager( tSongListView &lv, tMainWindow &mainwin, 
	const QString & pref_prefix );

    virtual void setup();
    void setupListView( const vector<bool> &visibility );

    tSongSet *songSet()
    { return SongSet; }
    void setSongSet( tSongSet *ss );

    virtual QPopupMenu *buildPopupMenu();

    void saveListViewAppearance( QSettings &settings );
    void loadListViewAppearance( QSettings &settings );

  public slots:
    void noticeSongModified( tSong *song );
    void noticeSongSetModified( bool substantial_change, tProgress *progress );

    void redisplay( bool preserve_scroll_pos, tProgress *progress = NULL );

    void itemRenamed( QListViewItem *item, int col  );

    void contextMenuRequested( QListViewItem *item, const QPoint &pos, int col );

    void slotPlayNow();
    void slotPlayNow( QListViewItem *item );
    void slotPlayNext();
    void slotPlayEventually();
    void slotPlayRandomSongs();

    void rateNone() { rate( -1 ); } 
    void rate0() { rate( 0 ); } 
    void rate1() { rate( 1 ); } 
    void rate2() { rate( 2 ); } 
    void rate3() { rate( 3 ); } 
    void rate4() { rate( 4 ); } 
    void rate5() { rate( 5 ); } 
    void rate( int rating );

    void resetStatistics();

    void deleteFromDisk();
    void highlightCurrentSong();

    void tagArtistTitle();
    void tagTitleArtist();
    void redoRetag();
    void rereadTag();
    void stripTag();

    void removeSelected();

    void searchForward( tCriterion *crit, bool from_beginning );

    void highlight( tSong *song );

    void setDisplaySorting( tSongField field, bool ascending );

    void pluginActivated( int id );

    void selectionChanged();

    virtual void headerClicked( int column ) = 0;

  public:
    friend class tSongListViewItem;
};




class tEditableSongSetViewManager : public tSongSetViewManager
{
    Q_OBJECT

    typedef tSongSetViewManager super;

  public:
    tEditableSongSetViewManager( tSongListView &lv, tMainWindow &mainwin,
	const QString & pref_prefix );

    QPopupMenu *buildPopupMenu();

  public slots:
    void uploadSongSet();
    void downloadSongSet();
    void importM3U();

    void sortBy( tSongField field );
    void jumble();
    void reverse();

    void sortByArtist() { sortBy( FIELD_ARTIST ); }
    void sortByTitle() { sortBy( FIELD_TITLE ); }
    void sortByAlbum() { sortBy( FIELD_ALBUM ); }
    void sortByTrackNumber() { sortBy( FIELD_TRACKNUMBER ); }
    void sortByYear() { sortBy( FIELD_YEAR ); }
    void sortByGenre() { sortBy( FIELD_GENRE ); }

    void songsDropped( bool from_same_widget, tSong *onto, vector<tFilename> &songs );

    void headerClicked( int column );
};




class tAllSongsViewManager : public tSongSetViewManager
{
    Q_OBJECT

    typedef tSongSetViewManager super;

  public:
    tAllSongsViewManager( tSongListView &lv, tMainWindow &mainwin,
	const QString & pref_prefix );

    QPopupMenu *buildPopupMenu();

  public slots:
    void slotAdd();

    void headerClicked( int column );

  signals:
    void add( tSongList const & );
};




#endif
