#include "base.h"
#include <stdexcept>




bool hasAttribute( const char *name, const char **attributes )
{
  while ( *attributes && strcmp( name, *attributes ) != 0 )
    attributes += 2;

  if ( *attributes == 0 )
    return false;
  return true;
}




QString lookupAttribute( const char *name, const char **attributes )
{
  while ( *attributes && strcmp( name, *attributes ) != 0 )
    attributes += 2;

  if ( *attributes == 0 )
    throw runtime_error( QString2string( "Attribute not found: " + QString::fromUtf8( name, strlen( name ) ) ) );
  return QString::fromUtf8( attributes[1], strlen( attributes[1] ) );
}




string QString2string( QString const &str )
{
  if ( str == QString::null )
    return "<NULL>";
  else
    return str.latin1();
}




QString string2QString( string const &str )
{
  return str.c_str();
}




/*
namespace {
  struct tWCharT2UCS2Iconv {
    iconv_t cd;

    tWCharT2UCS2Iconv()
    {
       cd = iconv_open ( "UCS2", "WCHAR_T" );
       if (cd == (iconv_t) -1)
       {
	 if (errno == EINVAL)
	 {
	   cerr << "could not instantiate wchar_t -> ucs2 conversion." << endl;
	   abort();
	 }
	 else
	   perror ("iconv_open");
       }
    }
    ~tWCharT2UCS2Iconv()
    {
      if (iconv_close (cd) != 0)
	perror ("iconv_close");
    }
  } WCharT2UCS2Iconv;
}




QString wstring2QString( const wchar_t *str )
{
  char *inptr = const_cast< char * >( reinterpret_cast<const char *>( str ) );
  size_t inlength = wcslen( str );
  size_t inlength_bytes = inlength * sizeof( wchar_t );

  size_t destsize = inlength_bytes;
  char destbuffer[ destsize ];
  char *destptr = destbuffer;

  if ( iconv ( WCharT2UCS2Iconv.cd, &inptr, &inlength_bytes, &destptr, &destsize ) == (size_t) -1 )
  {
    throw runtime_error( "Error during wchar_t -> UCS2 conversion." );
  }

  unsigned short *ucs2 = (unsigned short *) destbuffer;
  ucs2[ inlength ] = 0;

  return QString::fromUcs2( ucs2 );
}
*/




vector<QString> QStringList2QStringvector( const QStringList &strlist )
{
  vector<QString> result;

  for ( unsigned i = 0; i < strlist.size(); i++ )
    result.push_back( strlist[ i ]  );

  return result;
}




QStringList QStringvector2QStringList( const vector<QString> &strlist )
{
  QStringList result;

  for ( unsigned i = 0; i < strlist.size(); i++ )
    result.push_back( strlist[ i ]  );

  return result;
}




ostream &operator <<( ostream &ostr, QString const &str )
{
  return ( ostr << QString2string( str ) );
}




void runtimeError( QString const &str )
{
  throw std::runtime_error( str.latin1() );
}




unsigned long hash_QString::operator() ( QString const &str ) const
{
  unsigned long h = 0;
  for ( unsigned i = 0;i < str.length();i++ )
    h = 5 * h + str[ i ].unicode();
  return h;
}





// tProfiler ------------------------------------------------------------------
tProfiler::tProfiler( const QString &name )
: Name( name )
{
  Time.start();
}




tProfiler::~tProfiler()
{
  cerr << "[profiler] " << Name << " -> " << Time.elapsed() << " ms." << endl;
}

