#include <xmmsctrl.h>
#include <unistd.h>
#include <cstdlib>
#include <stdexcept>
#include "utility/player.h"
#include "database/song.h"

#include "player.moc"




class tXMMSPlayer : public tPlayer
{
    int Session;
  public:
    tXMMSPlayer();
    void ensureRunning();
    const QString name()
    {
      return "XMMS";
    }
    void playNow( tSongList const &songlist );
    void playNext( tSongList const &songlist );
    void playEventually( tSongList const &songlist );
    void setPlayList( tSongList const &songlist );
    void getPlayList( vector<tFilename> &songs );
    tFilename currentFilename();
    bool isPlaying();
};





tXMMSPlayer::tXMMSPlayer()
{
  Session = 0;
}




void tXMMSPlayer::ensureRunning()
{
  if ( !xmms_remote_is_running( Session ) )
  {
    system( "xmms &" );
    unsigned tries = 5;

    while ( tries-- && !xmms_remote_is_running( Session ) )
      sleep( 1 );

    if ( !xmms_remote_is_running( Session ) )
      throw runtime_error( "failed to start xmms." );
  }
}




void tXMMSPlayer::playNow( tSongList const &songlist )
{
  ensureRunning();
  if ( songlist.size() == 0 )
    return;

  gint start_playlist_pos = xmms_remote_get_playlist_length( Session );
  gint playlist_pos = start_playlist_pos;
  FOREACH_CONST( first, songlist, tSongList )
  {
    xmms_remote_playlist_ins_url_string( Session, const_cast<char *>( ( *first )->filename().latin1() ), playlist_pos++ );
  }

  xmms_remote_set_playlist_pos( Session, start_playlist_pos );
  if ( !xmms_remote_is_playing( Session ) )
    xmms_remote_play( Session );
}




void tXMMSPlayer::playNext( tSongList const &songlist )
{
  ensureRunning();

  if ( songlist.size() == 0 )
    return;

  gint playlist_pos = xmms_remote_get_playlist_pos( Session );
  if ( xmms_remote_get_playlist_length( Session ) == 0 )
    playlist_pos = -1;
  FOREACH_CONST( first, songlist, tSongList )
  {
    xmms_remote_playlist_ins_url_string( Session, const_cast<char *>( ( *first )->filename().latin1() ), ++playlist_pos );
  }
}




void tXMMSPlayer::playEventually( tSongList const &songlist )
{
  ensureRunning();

  if ( songlist.size() == 0 )
    return;

  gint playlist_pos = xmms_remote_get_playlist_length( Session );
  FOREACH_CONST( first, songlist, tSongList )
  {
    xmms_remote_playlist_ins_url_string( Session, const_cast<char *>( ( *first )->filename().latin1() ), playlist_pos++ );
  }
}




void tXMMSPlayer::setPlayList( tSongList const &songlist )
{
  ensureRunning();
  xmms_remote_playlist_clear( Session );
  gint playlist_pos = 0;
  FOREACH_CONST( first, songlist, tSongList )
  {
    xmms_remote_playlist_ins_url_string( Session, 
	const_cast<char *>( ( *first )->filename().latin1() ), 
	playlist_pos++ );
  }
}




void tXMMSPlayer::getPlayList( vector<tFilename> &songs )
{
  ensureRunning();

  songs.clear();

  int length = xmms_remote_get_playlist_length( Session );
  for ( int i = 0; i < length; i++ )
    songs.push_back( xmms_remote_get_playlist_file( Session, i ) );
}




tFilename tXMMSPlayer::currentFilename()
{
  return xmms_remote_get_playlist_file( Session, 
      xmms_remote_get_playlist_pos( Session ) );
}




bool tXMMSPlayer::isPlaying()
{
  return xmms_remote_is_playing( Session ) && 
    !xmms_remote_is_paused( Session );
}



// factory interface ----------------------------------------------------------
void listPlayers( vector<QString> &players )
{
  players.clear();
  players.push_back( "XMMS" );
}




tPlayer *createPlayer( QString const &name )
{
  if ( name == "XMMS" )
  {
    return new tXMMSPlayer;
  }
  else
    return NULL;
}
